//
// SLTMSoundConsts.h - Constands and structures for sound support
//	WARNING - shared files between Host and Target software
//
// started 11.11.2001 as part of COD2 project by Nik
// modified 07.06.2004 version 0x0107 by Shish add spdif support
// modified 30.11.2004 version 0x0108 by Shish restore to life 
// 			the Output sound buffer bypass WDM driver
// modified 10.02.2005 version 0x0109 by Shish add:
//			pass at least one (or any of 3) wave output to 
//			any output channel 
// modified 25.05.2005 version 0x0109 by Shish add:
//			   WDM .
//			     COD.out  
//			 SoundDriverData     DMA
//			(     PCI)
// 


#ifndef SLTM_SOUNDCONSTS_H
#define SLTM_SOUNDCONSTS_H

#define TMC_Sound_Version   (0x0109)

typedef enum {
    tmc_sound_get_Status,
    tmc_sound_put_MixVolume,
    tmc_sound_get_MixVolume,
    tmc_sound_put_OutVolume,
    tmc_sound_get_OutVolume,
    tmc_sound_put_InVolume,
    tmc_sound_get_InVolume,
    tmc_sound_put_DelayMs,
    tmc_sound_get_DelayMs,
    tmc_sound_Start,  // param: 1 ULONG SampleRate
    tmc_sound_Stop,  //
    tmc_sound_get_Ratio,  //

	// begin unsupported commands (v0x0104)
	// begin restored commands (v0x0108) for non WDM playback with sync to Video
    tmc_sound_BufferGetGranularity,  // param: [out] long irqSizeByte
    tmc_sound_BufferSet,  // param: long Size, long MemHandle, [out] long err
    tmc_sound_BufferStart,
    tmc_sound_BufferStop,
    tmc_sound_BufferRelease,
	// end restored commands (v0x0108)
    tmc_sound_InBufferSet,  // param: long Size, long MemHandle, [out] long err
    tmc_sound_InBufferStart,
    tmc_sound_InBufferStop,
    tmc_sound_InBufferRelease,
	// end unsupported commands (v0x0104)

    tmc_sound_put_MonoMixVolume,
    tmc_sound_get_MonoMixVolume,
    tmc_sound_put_MonoOutVolume,
    tmc_sound_get_MonoOutVolume,
    tmc_sound_put_MonoInVolume,
    tmc_sound_get_MonoInVolume,
    tmc_sound_put_MixMute,
    tmc_sound_get_MixMute,
    tmc_sound_put_OutMute,
    tmc_sound_get_OutMute,
    tmc_sound_put_InMute,
    tmc_sound_get_InMute,
    tmc_sound_put_InBoost,
    tmc_sound_get_InBoost,
    tmc_sound_get_Version,
    tmc_sound_OpenMeters,
    tmc_sound_CloseMeters,
    tmc_sound_get_Meters,
	tmc_sound_get_AllParam,
	// v0x0108
	tmc_sound_get_FrameRate, // get video frame rate return: param[2] = rate, param[3] = scale. 25000 * 2/1000 - PAL, 3000*3/1001 - NTSC
	// v0x0109
	tmc_sound_put_WaveOutCross,
	tmc_sound_get_WaveOutCross,
} sltm_sound_commands;

typedef enum eVolume {
        NormalVolumeShift = 12
        //,NormalVolume = 1<<NormalVolumeShift
        ,NormalVolume = 0
        ,MaxVolumeShift = NormalVolumeShift+2
        //,MaxVolume = 1<<MaxVolumeShift
        ,MaxVolume = 120
        ,MinVolume = -960
        ,HrdAttenShift = 7
        ,HrdAtten = 1<<HrdAttenShift
        ,MaxInChannels = 6
        ,MaxOutChannels = 6
		,ChannelsPerLine = 2
		,MaxOutLines = (MaxOutChannels / ChannelsPerLine)
} sltm_sound_volume;

typedef enum SLTMSndStatus_t {
    eSndStatusFail = -1,    // Sound device not present!!!!
    eSndStatusOk = 0,       // Sound device present and stopped
    eSndStatusOpened = 1,   // Sound device present and opened (it's interim trimedia state)
    eSndStatusStarted = 2,  // Sound device present and audio thru is started
							// it's set the Granularity member of 'SoundDriverData' structure
    eSndCommandSetFifos = 3 // Must be sended after status(eSndStatusStarted) successfuly
							// sets and before start any operations with the fifos.
							// Do not change m_Global.m_State
} SLTMSndStatus;


#ifndef FIELD_OFFSET
#define FIELD_OFFSET(type, field)    ((long)&(((type *)0)->field))
#endif

////////////////////////////////////////////////////////////////////////////////
// ALERT ALERT ALERT - ALL structures MUST correspond to the HOST structures
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// the fifo header for playback bypass WDM driver
//        
//             ,
//     ().  -      (+/-1) .. 
//  -          
//	 :
// offset				|	field			|	comment
//--------------------------------------------------------------------
//	0					|	blocks			|	beginning of SoundOutFifoHeader -\
//						|	...				|									  > sizeof(SoundOutFifoHeader)
//______________________|___________________|	ending of SoundOutFifoHeader	-/
//	firstBlockOffset	|					|	beginning of SoundOutFifoBlockHeader-\
//						|	...				|									  	  > sizeof(SoundOutFifoBlockHeader)
//______________________|___________________|	ending of SoundOutFifoBlockHeader   -/
//______________________|___block Data______|
//	firstBlockOffset +	|					|	beginning of SoundOutFifoBlockHeader-\
//	blockSize			|	...				|									  	  > sizeof(SoundOutFifoBlockHeader)
//						|					|	ending of SoundOutFifoBlockHeader   -/
//______________________|___block Data______|
// .............

// the fields writed from Host (PC) under initialization
typedef struct SoundOutFifoStableHeader {
	long numberOfBlocks;		// number of bocks
	long blockSize;				// size of block with BlockHeader (in bytes)
	long maxDataSize;			// size of block data (in bytes)
	float samplesPerFrame;		//  -   
	long firstBlockOffset;		// offset for the beginning of first block (in bytes)
} SoundOutFifoStableHeader;

typedef enum EDecreaseConsts {
	eDecreaseLimit	= 0x00ffffff,
	eDecreaseScale	= 0x0000ffff
} EDecreaseConsts;

// the fields writed from Host (PC) only
typedef struct SoundOutFifoFromHostHeader {
    long nextWriteBlock;			//           >= 0 and < numberOfBlocks
    unsigned long writedFrames;    	// low part of      
	long writedFramesHi;			// Hi part of writedFrames 
} SoundOutFifoFromHostHeader;

// the fields writed from Trimedia only
typedef struct SoundOutFifoFromTMHeader {
	unsigned long currVideoFrame;	// the current absolute video frame number
	long currVideoFrameHi;			// Hi part of current video frame
	long readedBlock;				//     >= 0 and < numberOfBlocks
	unsigned long readedFrames;    	// low part of      
	long readedFramesHi;			// Hi part of readedFrames
} SoundOutFifoFromTMHeader;

typedef struct SoundOutFifoBlockHeader {
	unsigned long videoFrame;   //          
	long videoFrameHi;			//            
	long lengthOfBlock;			//     
} SoundOutFifoBlockHeader;

typedef struct SoundOutFifoHeader {
	// the fields writed from Host (PC) under initialization
	SoundOutFifoStableHeader _stableHeader;
	char reserved1[64 - sizeof(SoundOutFifoStableHeader)];

	// the fields writed from Host (PC) only
	SoundOutFifoFromHostHeader _fromHost;
	char reserved2[64 - sizeof(SoundOutFifoFromHostHeader)];

	// the fields writed from Trimedia only
	SoundOutFifoFromTMHeader _fromTM;		// the current absolute video frame number
	char reserved3[64 - sizeof(SoundOutFifoFromTMHeader)];
} SoundOutFifoHeader;

#define OutFifoFromHostOffset  	FIELD_OFFSET(SoundOutFifoHeader, _fromHost)

#define OutFifoFromTMOffset   	FIELD_OFFSET(SoundOutFifoHeader, _fromTM)

//-----------------------------------------------------------------------------
// Struct for Volume mix & e.t.c.
//   WDM      ISLTMSound  SLTMSrvd.dll

typedef struct SoundParameters_t {
	long m_SampleRate;
	long m_Status;
	long m_MixVolume[MaxInChannels][MaxOutChannels];
	long m_MixMute[MaxInChannels][MaxOutChannels];
	long m_DelayMs[MaxInChannels];
    long m_InVolume[MaxInChannels];
    long m_InMute[MaxInChannels];
    long m_InBoost[MaxInChannels/2];
    long m_OutVolume[MaxOutChannels];
    long m_OutMute[MaxOutChannels];
	long m_bAVSync;
	// v0x0109
	long m_WaveOutCross[MaxOutLines][MaxOutLines];
} SoundParameters;

//-----------------------------------------------------------------------------
// Constants and struct for WDM Audio Driver
//-----------------------------------------------------------------------------

#define SndMaxOutFifos	(MaxOutLines)

typedef struct SoundFifoHeader_t {
    long Size;      // size of fifo data (in bytes)
    long Offset;    // offset of fifo data (in bytes)
	char reserved1[64];
    long ReadPos;   // absolute offset of reading data must be >= Offset and < (Offset+Size)
    unsigned long Readed;    // absolute counter of readed bytes in fifo
	char reserved2[64];
    long WritePos;  // absolute offset of writinging data must be >= Offset and < (Offset+Size)
    unsigned long Writed;    // absolute counter of writed bytes in fifo
	char reserved3[64];
} SoundFifoHeader;

typedef enum eSLTMFifoState_t {
	eSndFifoNone = 0,		
	eSndFifoStop,
	eSndFifoStarting,
	eSndFifoPause,
	eSndFifoRun
} eSLTMFifoState;

// new in v(0x0106) 19.05.2004
typedef enum eSLTMAudioDeviceConfiguration {
	eSndConfigNormal = 0		// normal audio device - TriMedia in Master mode
								// 3 stereo analog out and 3 stereo analog input with CrossMix
	,eSndConfigSingle			// reduced audio device - 1 stareo analog out & 1 stereo analog input
								//   ,   , Single     Normal,
								//  WDM    -  (    )
	,eSndConfigDigital			// reduced audio device -    Slave ,    
								// in1 - SPDIF input from CS4226
								// in2 & in3    
								//    
								// WDM      
} eSLTMAudioDeviceConfiguration;

// new in v(0x0107) 03.06.2004
typedef enum eSLTMAudioSPDOType {
	eSndSPDOConsumerType = 0
	,eSndSPDOProfessionalType = 1
} eSLTMAudioSPDOType;

typedef struct SoundGlobalCommand_t {
	volatile long m_Count; 			// Global command counter
	volatile long m_State;			// eSLTMFifoState Global State == SLTMSndStatus
	volatile long m_SampleRate;  	// Global SampleRate
	volatile long m_DeviceConfig;	// Device configuration		new in v(0x0106) 19.05.2004
	volatile long m_SPDOType;		// SPDIF output type (pro/consumer)	new in v(0x0107) 03.06.2004
} SoundGlobalCommand;

typedef struct SoundFifoCommand_t {
	long m_Count;          			// Fifo command counter
	volatile long m_State;			// Fifo State
} SoundFifoCommand;

typedef struct SoundCommand_t {
	unsigned long m_VolCount;		// Volume command counter
	SoundParameters m_Data;   		// Volume data
	SoundGlobalCommand m_Global;   	// Global command
	SoundFifoCommand m_OutFifo[SndMaxOutFifos];      // Out fifo command
	SoundFifoCommand m_InFifo;      // In fifo command
} SoundCommand;

typedef struct SoundDriverData_t {
	char reserved0[64];
	long m_cb;	  					// struct's length - sets from trimedia
	unsigned long m_Granularity;	// sets from trimedia after Start command is finished
	char reserved01[64];
	unsigned long m_hId;			// DMABuffer handle: sets from PC with 'eSndCommandSetFifos'
									// command is set
	unsigned long m_bDmaReady;		//   cod.out   
	char reserved1[64];
	SoundCommand m_FromPC;			//      
	char reserved2[64];
	SoundCommand m_FromTri;			//       
	char reserved3[64];
	SoundFifoHeader m_OutHdr[SndMaxOutFifos];	//    (Out)
	SoundFifoHeader	m_InHdr;		//    (In)
	char reserved4[64];
} SoundDriverData;

#endif /* SLTM_SOUNDCONSTS_H */
