// simpgrab.h : IDL source for ISimpleGrabber interface

typedef [v1_enum] enum {
	SG_Background,
	SG_MixOut,
	SG_Artist,
	SG_Artist_1,
	SG_Artist_2
} SGSource;

typedef [v1_enum] enum {
	SG_None,
	SG_Free,
	SG_Busy
} SGStatus;

typedef [v1_enum] enum {
	SG_UYVY,	// (U Y V Y) blocks
	SG_BGRA,
	SG_RGBA,
	SG_BGR,
	SG_RGB,
	SG_YUV16,
	SG_YUVA16,
	SG_YUY2,    // (Y U Y V) blocks
	SG_UYVY_A,	// split image horizontally ...
	SG_YUY2_A,	// ... left part - image, right - alpha (16..240 - full range)
	SG_YV12,    // 4:2:0 (all Y) (all V) (all U)
	SG_IYUV,    // 4:2:0 (all Y) (all U) (all V), the same as I420
} SGPixelFormat;

typedef [v1_enum] enum {
	SG_Interlaced = 0,
	SG_Progressive,
	SG_UpperFieldOnly
} SGFieldsType;

typedef [v1_enum] enum {
	SG_Top		= 0,	// top or progressive
	SG_Bottom	= 1		// bottom only
} SGFieldPosition;

typedef [v1_enum] enum {
	SG_MinFieldNumberInStream	= 0x0008,
	SG_MaxFieldNumberInStream	= 0x7FF0,
	SG_UseLastField				= 0x7FFE,
	SG_SkipAllFields			= 0x7FFF,
	SG_SkipAllExceptLastField	= 0x7FFE
} SGSkipFildType;

[
	object,
	uuid(AB6AE8B5-2D78-435F-BA5F-66E84A48C545),
	helpstring("ISimpleGrabberOld Interface"),
	pointer_default(unique)
]
interface ISimpleGrabberOld : IUnknown
{
	[helpstring("method GetBoardCount returns number of available TriMedia boards")] HRESULT GetBoardCount([out] long *BoardsCount);
	[helpstring("method GetBoardStatus returns staus and driver+hardware version of specified TriMedia board")] HRESULT GetBoardStatus([in] long BoardIndex, [out] SGStatus *Status, [out] long *Version);
	[helpstring("method AcquireBoard tries to lock specified TriMedia board")] HRESULT AcquireBoard([in] long BoardIndex);
	[helpstring("method FreeBoard unlocks previously locked TriMedia board")] HRESULT FreeBoard();
	[helpstring("method GetBoardInfo returns video parameters")] HRESULT GetBoardInfo([out] long* dx, [out] long* dy, [out] double* FrameRate);
	[helpstring("method GetBoardFieldNumber returns value of hardware field counter")] HRESULT GetBoardFieldNumber([out] long *HardwareFieldNumber);
	[helpstring("method SwitchChannel switchs source for streaming")] HRESULT SwitchChannel([in] SGSource Src);
	[helpstring("method CaptureField capture one field only to specified buffer")] HRESULT CaptureField([in] SGSource Src, [out] SGFieldPosition *FieldPos, [in] SGPixelFormat Format, [out, size_is(cb)] BYTE* VideoData, [in] long cb, [in] long stride);
	[helpstring("method StartStream create fifo and start capturing video data")] HRESULT StartStream([in] SGSource Src, [in] long FifoSizeInFields, [in] long UseColorCorrection);
	[helpstring("method StopStream capturing of video data and clear fifo")] HRESULT StopStream();
	[helpstring("method GetStreamInfo returns info about current state of fifo")] HRESULT GetStreamInfo([out] long *NumberOfCapturedFields, [out] long *NumberOfFreeFields, [out] long *FirstFieldNumber, [out] long *LastFieldNumber);
	[helpstring("method SkipFields relases space into fifo for next field capturing")] HRESULT SkipFields([in] long SkipCount);
	[helpstring("method GetFieldInfo returns info about specified captured field")] HRESULT GetFieldInfo([in] long FieldIndex, [out] long* FieldNumber, [out] SGSource *Src, [out] SGFieldPosition *FieldPos);
	[helpstring("method GetFieldImage convert/copy captured image to specified buffer")] HRESULT GetFieldImage([in] long FieldIndex, [in] long x0, [in] long y0, [in] long dx, [in] long dy, [in] SGPixelFormat Format, [out, size_is(cb)] BYTE* VideoData, [in] long cb, [in] long stride);
	[helpstring("method Debug prints to OutputDebugString some statistic info")] HRESULT Debug();
};
[
	object,
	uuid(D54DB115-BD03-4cb8-81F0-0258EA459BC6),
	helpstring("ISimpleGrabber Interface"),
	pointer_default(unique)
]
interface ISimpleGrabber : IUnknown
{
	[helpstring("method GetBoardCount returns number of available TriMedia boards")] HRESULT GetBoardCount([out] long *BoardsCount);
	[helpstring("method GetBoardStatus returns staus and driver+hardware version of specified TriMedia board")] HRESULT GetBoardStatus([in] long BoardIndex, [out] SGStatus *Status, [out] long *Version);
	[helpstring("method AcquireBoard tries to lock specified TriMedia board")] HRESULT AcquireBoard([in] long BoardIndex);
	[helpstring("method FreeBoard unlocks previously locked TriMedia board")] HRESULT FreeBoard();
	[helpstring("method GetBoardInfo returns video parameters")] HRESULT GetBoardInfo([out] long* dx, [out] long* dy, [out] double* FrameRate);
	[helpstring("method GetBoardFieldNumber returns value of hardware field counter")] HRESULT GetBoardFieldNumber([out] long *HardwareFieldNumber);
	[helpstring("method SwitchChannel switchs source for streaming")] HRESULT SwitchChannel([in] SGSource Src);
	[helpstring("method CaptureField capture one field only to specified buffer")] HRESULT CaptureField([in] SGSource Src, [out] SGFieldPosition *FieldPos, [in] SGPixelFormat Format, [out, size_is(cb)] BYTE* VideoData, [in] long cb, [in] long stride);
	[helpstring("method StartStream create fifo and start capturing video data")] HRESULT StartStream([in] SGSource Src, [in] long FifoSizeInFields, [in] long UseColorCorrection);
	[helpstring("method StopStream capturing of video data and clear fifo")] HRESULT StopStream();
	[helpstring("method GetStreamInfo returns info about current state of fifo")] HRESULT GetStreamInfo([out] long *NumberOfCapturedFields, [out] long *NumberOfFreeFields, [out] long *FirstFieldNumber, [out] long *LastFieldNumber);
	[helpstring("method SkipFields relases space into fifo for next field capturing")] HRESULT SkipFields([in] long SkipCount);
	[helpstring("method GetFieldInfo returns info about specified captured field")] HRESULT GetFieldInfo([in] long FieldIndex, [out] long* FieldNumber, [out] SGSource *Src, [out] SGFieldPosition *FieldPos, [out] long* dx, [out] long* dy);
	[helpstring("method GetFieldImage convert/copy captured image to specified buffer")] HRESULT GetFieldImage([in] long FieldIndex, [in] long x0, [in] long y0, [in] long dx, [in] long dy, [in] SGPixelFormat Format, [out, size_is(cb)] BYTE* VideoData, [in] long cb, [in] long stride);
	[helpstring("method Debug prints to OutputDebugString some statistic info")] HRESULT Debug();
	[helpstring("method GetFrameImage convert/copy captured image to specified buffer")] HRESULT GetFrameImage([in] long UpperFieldIndex, [in] long LowerFieldIndex, [in] long x0, [in] long y0, [in] long dx, [in] long dy, [in] SGPixelFormat Format, [out, size_is(cb)] BYTE* VideoData, [in] long cb, [in] long stride);
	[helpstring("method CaptureFields captures specified number of fields to fifo")] HRESULT CaptureFields([in] SGSource Src, [in] long FieldsCount, [in] long UseColorCorrection);
	[helpstring("method CaptureFrame capture one frame to specified buffer")] HRESULT CaptureFrame([in] SGSource Src, [in] SGPixelFormat Format, [out, size_is(cb)] BYTE* VideoData, [in] long cb, [in] long stride);
};

[
	object,
	uuid(753849B3-A8AF-4e4f-9990-107871BD3DF1),
	helpstring("ISimpleGrabber2 Interface"),
	pointer_default(unique)
]
interface ISimpleGrabber2 : IUnknown
{
	[helpstring("method GetBoardInfo2 returns video parameters")] HRESULT GetBoardInfo2([out] long* field_width, [out] long* field_height, [out] double* field_rate, [out] SGFieldsType* field_type);
};


[
	object,
	uuid(7DF7B066-CFC2-4ea8-83BC-AC172017631C),
	helpstring("ISimpleGrabberMS Interface"),
	pointer_default(unique)
]
interface ISimpleGrabberMS : IUnknown
{
	[helpstring("method SetMasterMode")]   HRESULT SetMasterMode([in] long FifoSizeInBytes, [out] IUnknown** ppILockedMemoryObject);
	[helpstring("method SetSlaveMode")]    HRESULT SetSlaveMode([in] long BoardIndex, [in] IUnknown* pILockedMemoryObject);
	[helpstring("method ResyncWithFifo")]  HRESULT ResyncWithFifo();
	[helpstring("method GetFreeAndUsed")]  HRESULT GetFreeAndUsed([out] long *pFreeInBytes, [out] long *pUsedInBytes);
}


typedef struct VITC_TimeCode_t {
	BYTE frames;	// hex-decimal format
	BYTE seconds;	// hex-decimal format
	BYTE minutes;	// hex-decimal format
	BYTE hours;		// hex-decimal format
	BYTE user_bits[4];
} VITC_TimeCode;

[
	object,
	uuid(01D53849-B777-443d-B4A0-4F6C2A793FE5),
	helpstring("ISimpleGrabberGetVITC Interface"),
	pointer_default(unique)
]
interface ISimpleGrabberGetVITC : IUnknown
{
	[helpstring("method GetVITC returns raw VITC information (8 bytes) from specified captured field")] HRESULT GetVITC([in] long FieldIndex, [out] VITC_TimeCode* VITC_array);
}


typedef [v1_enum] enum {
	SG_Video1	= 0x01,			// video stream #1
	SG_Video2	= 0x02,			// video stream #2
	SG_Alpha	= 0x04,
	SG_Audio	= 0x08,
	SG_VBI_OLD			= 0x10,
	SG_Video4Artist_1	= 0x20,	// video from decoder #1 (LineA)
	SG_Video4Artist_2	= 0x40,	// video from decoder #2 (LineB)
    SG_VBI_LINE_1 = 0x00000100,
    SG_VBI_LINE_2 = 0x00000200,
    SG_VBI_LINE_3 = 0x00000400,
    SG_VBI_LINE_4 = 0x00000800,
    SG_VBI_LINE_5 = 0x00001000,
    SG_VBI_LINE_6 = 0x00002000,
    SG_VBI_LINE_7 = 0x00004000,
    SG_VBI_LINE_8 = 0x00008000,
	SG_VBI_MASK	  = 0x0000FF00
} SGFieldDataType;

[
	object,
	uuid(88E52AD0-238A-4b8b-B910-6A5A917170BB),
	helpstring("ISimpleGrabberData Interface"),
	pointer_default(unique)
]
interface ISimpleGrabberData : IUnknown
{
	[helpstring("method StartStreamAdv create fifo and start capturing video and other data (SGFieldDataType)")] HRESULT StartStreamAdv([in] SGSource Src, [in] long FifoSizeInFields, [in] long UseColorCorrection, [in] long enable_data_bits);
	[helpstring("method GetFieldDataInfo returns bit mask of existing data types")] HRESULT GetFieldDataInfo([in] long FieldIndex, [out] SGFieldDataType* data_bits);
	[helpstring("method GetVBIData copy specified line")] HRESULT GetVBIData([in] long FieldIndex, [in] SGFieldDataType vbi_type, [out, size_is(cb)] BYTE* VBIData, [in] long cb);
	[helpstring("method GetAudioDataInfo returns audio format & size")] HRESULT GetAudioDataInfo([in] long FieldIndex, [out] long* sample_count, [out] long* sample_rate, [out] long* channel_count, [out] long* complete_sample_size);
	[helpstring("method GetAudioData copy all audio data")] HRESULT GetAudioData([in] long FieldIndex, [out, size_is(cb)] BYTE* AudioData, [in] long cb);
	[helpstring("method GetFieldImageAdv convert/copy captured field image to specified buffer")] HRESULT GetFieldImageAdv([in] long FieldIndex, [in] long x0, [in] long y0, [in] long dx, [in] long dy, [in] SGPixelFormat Format, [out, size_is(cb)] BYTE* VideoData, [in] long cb, [in] long stride, [in, defaultvalue(SG_Video1)] SGFieldDataType imageType);
	[helpstring("method GetFrameImageAdv convert/copy captured frame image to specified buffer")] HRESULT GetFrameImageAdv([in] long UpperFieldIndex, [in] long LowerFieldIndex, [in] long x0, [in] long y0, [in] long dx, [in] long dy, [in] SGPixelFormat Format, [out, size_is(cb)] BYTE* VideoData, [in] long cb, [in] long stride, [in, defaultvalue(SG_Video1)] SGFieldDataType imageType);
	[helpstring("method GetFieldImageAsDIB convert/copy captured field image to specified bitmap")] HRESULT GetFieldImageAsDIB([in] long FieldIndex, [in] long x0, [in] long y0, [in, size_is(cbBmpInfo)] BYTE *pBmpInfo, [in] long cbBmpInfo, [out, size_is(cbBmpData)] BYTE* pBmpData, [in] long cbBmpData, [in, defaultvalue(SG_Video1)] SGFieldDataType imageType);
	[helpstring("method GetFrameImageAsDIB convert/copy captured frame image to specified bitmap")] HRESULT GetFrameImageAsDIB([in] long UpperFieldIndex, [in] long LowerFieldIndex, [in] long x0, [in] long y0, [in, size_is(cbBmpInfo)] BYTE *pBmpInfo, [in] long cbBmpInfo, [out, size_is(cbBmpData)] BYTE* pBmpData, [in] long cbBmpData, [in, defaultvalue(SG_Video1)] SGFieldDataType imageType);
	[helpstring("method GetFieldImageAs420Planar convert/copy captured field image to specified buffer by fields (strideUV=strideY/2, cbUV=strideUV*dy/2)")] HRESULT GetFieldImageAs420Planar([in] long FieldIndex, [in] long x0, [in] long y0, [in] long dx, [in] long dy, [out, size_is(cbY)] BYTE* DataY, [in] long cbY, [in] long strideY, [out, size_is(cbU)] BYTE* DataU, [in] long cbU, [in] long strideU, [out, size_is(cbV)] BYTE* DataV, [in] long cbV, [in] long strideV, [in, defaultvalue(SG_Video1)] SGFieldDataType imageType);
	[helpstring("method GetFrameImageAs420Planar convert/copy captured frame image to specified buffer by fields (strideUV=strideY/2, cbUV=strideUV*dy/2)")] HRESULT GetFrameImageAs420Planar([in] long UpperFieldIndex, [in] long LowerFieldIndex, [in] long x0, [in] long y0, [in] long dx, [in] long dy, [out, size_is(cbY)] BYTE* DataY, [in] long cbY, [in] long strideY, [out, size_is(cbU)] BYTE* DataU, [in] long cbU, [in] long strideU, [out, size_is(cbV)] BYTE* DataV, [in] long cbV, [in] long strideV, [in, defaultvalue(SG_Video1)] SGFieldDataType imageType);
	[helpstring("method GetFieldImageAs420Planar2 convert/copy captured field image to specified buffer by frame (strideUV=strideY/2, cbUV=strideUV*dy/2)")] HRESULT GetFieldImageAs420Planar2([in] long FieldIndex, [in] long x0, [in] long y0, [in] long dx, [in] long dy, [out, size_is(cbY)] BYTE* DataY, [in] long cbY, [in] long strideY, [out, size_is(cbU)] BYTE* DataU, [in] long cbU, [in] long strideU, [out, size_is(cbV)] BYTE* DataV, [in] long cbV, [in] long strideV, [in, defaultvalue(SG_Video1)] SGFieldDataType imageType);
	[helpstring("method GetFrameImageAs420Planar2 convert/copy captured frame image to specified buffer by frame (strideUV=strideY/2, cbUV=strideUV*dy/2)")] HRESULT GetFrameImageAs420Planar2([in] long UpperFieldIndex, [in] long LowerFieldIndex, [in] long x0, [in] long y0, [in] long dx, [in] long dy, [out, size_is(cbY)] BYTE* DataY, [in] long cbY, [in] long strideY, [out, size_is(cbU)] BYTE* DataU, [in] long cbU, [in] long strideU, [out, size_is(cbV)] BYTE* DataV, [in] long cbV, [in] long strideV, [in, defaultvalue(SG_Video1)] SGFieldDataType imageType);
}

typedef [v1_enum] enum {
	SGVersion_1_00	= 0x0100,	// old version - before FTS_4_0 (FTS_3_10 and before)
	SGVersion_1_01	= 0x0101,	// add SG_Video4Artist_1 & SG_Video4Artist_2 support
	SGCurrentVersionPlus1
} SGVersion;

[
	object,
	uuid(121A2D55-34B3-420e-B112-52440C196BF1),
	helpstring("ISimpleGrabberVersion Interface"),
	pointer_default(unique)
]
interface ISimpleGrabberVersion : IUnknown
{
	[helpstring("method GetVersion return version of object")] HRESULT GetVersion([out] long* pVersion);
}
