/*
 *
 *
 * $Id: sltmPack.h,v 1.2 2001/08/16 07:58:36 nik Exp $
 */
/*
 *                 packHeader
 * /-------------------^----------------------\
 * |n| packItem[0] | packItem[1] | packItem[2] | ....... | itemData0 |...| itemData1 | ..... |
 * |<-----------------Offset0--------------------------->|<--size0-->|
 * |<------------------------Offset1------------------------------------>|<--size1-->|
 *
 */

#ifndef SLTMPACK_H
#define SLTMPACK_H

#include "TMDriverConsts.h"

/**
 *  y    FIFO     .
 */
typedef struct sltmFifoHeader {
	/**
     * offset to data begin (int bytes)
     */
	unsigned long offset;
	/**
     * data size (in bytes)
     */
	unsigned long size;
	/**
     *   
     */
	unsigned long sizeOfThis;
	/**
     *  FIFO -     TriMeda   
     * @since 2003-09-01
     */
	unsigned long fifo_state;
	/**
     * TriMEdia      FIFO.   .
     * @since 2003-10-01
     */
	unsigned long start_fifo;
	unsigned char dummy0[64 - 5 * 4];
	/**
     * number of filled data from absolut start (in bytes)
     */
	unsigned long filled;
	unsigned char dummy1[64 - 1 * 4];
	/**
     * number of readed data from absolut start (in bytes)
     */
	unsigned long readed;
	/**
     * reading position into fifo [0..size-1]
	 * should be locked by external mutex to correspond 'readed'
     */
	unsigned long readPosition;
	/**
     */
	unsigned char dummy2[64 - 3 * 4];
	/**
     *   0     . 
     *       .
     */
	unsigned long checksum;
} sltmFifoHeader_t;

/**
 *    FIFO.     
 *   
 */
enum eSltmPackHeaderKeys {
    /**
     *   
     */
    esphkSelf = 0,
    /**
     *   
     * @see sltmVideoFormat
     */
    esphkFormat = 1,
    /**
     *  Y  
     */
    esphkY = 2,
    /**
     *  U
     */
    esphkU = 3,
    /**
     *  V
     */
    esphkV = 4,
    /**
     *  
     */
    esphkSound = 5,
    /**
     *    VBI 
     */
    esphkVBI = 6
};

/**
 *     FIFO
 */
typedef struct sltmPackItem {
    /**
     *       
     */
	long offset;
    /**
     * . 0 
     */
	long size;
} sltmPackItem_t;

#define SLTM_PACKET_CHUNK_NUMBER 8
/**
 *    FIFO   . 
 * 
 */
typedef struct sltmPackHeader {
    /**
     *    
     */
	long n;
    /**
     *      
     *   == 0 -   
     * 
     * @see eSltmPackHeaderKeys
     */
	sltmPackItem_t items[SLTM_PACKET_CHUNK_NUMBER];
} slPackHeader_t;

typedef enum sltmFieldHeaderFlags {
	/**
     *     -  XILINX  
     *  
     * 
     * @see coFieldFlags
     */
	sltmValidTimestamp = 0x00000001,
	sltmPacketContainsError = 0x00000002,
	sltmValidDts = 0x00000004,
    /**
     *   
     */
	sltmField2 = 0x00000008,
    /**
     *   
     *   
     */
	sltmLowerField = 0x00000008,	
	sltmPacketContainsStartCode = 0x00000010,
    /**
     *      
     */
	sltmLiveVideo = 0x00000020,
    /**
     *       
     */ 
    sltmOldField  = 0x00000040
} sltmFieldHeaderFlags_t;

typedef struct sltmTimeStamp {
	unsigned long ticks;
	unsigned long hiTicks;
} sltmTimeStamp_t, *pisltmTimeStamp_t;

/**
 *      FIFO
 */
typedef struct sltmVideoFormat {
	CSLTMRGBAlfaFormat RGBAlphaFormat;
	CSLTMVideoStandart videoStandard;
	long imageWidth;
	long imageHeight;
	long imageStride;
	long imageUVStride;
	/**
     *      VITC
     */
	long vitcLO;
	long vitcHI;
	long activeVideoEndX;
	long activeVideoEndY;
    /**
     * 
     * @see sltmFieldHeaderFlags
     */
	unsigned long flags;
	/**
     *   
     */
	sltmTimeStamp_t time;
} sltmVideoFormat_t;

typedef struct streamField {
	long bufferNumber;
	long fieldNumber;
	sltmTimeStamp_t time;
} streamField_t;

typedef struct streamHeader {
	long status;
	streamField_t validField;
	streamField_t currentField;
} streamHeader_t;

#define STRIDE_MASK (63)
#define MEM_BLOCK_SIZE	(512)
#define MEM_BLOCK_MASK	(511)
#define MEM_PAGE_SIZE	(4*1024)
#define MEM_PAGE_MASK	(4*1024 - 1)


/**
 * ID       VBI 
 * 
 * @see sVBIChunk
 */
typedef enum eVBIChunkID {
    /**
     * Closed Caption
     */
    evbiCC = 1,
    /**
     * VITC
     */
    evbiVITC = 2
} eVBIChunkID_t;

/**
 *      FIFO   VBI 
 *   VBI       esphkY
 * ,   SLTM_F16YUV      16Y8U8V  format.imageStride
 * 
 * @see esphkVBI
 */
typedef struct sVBIChunk {
    /**
     *  0 id          idx
     * 
     * @see eVBIChunkID
     */
    unsigned char  id_list[16];
} sVBIChunk_t;

/**
 *      FIFO   
 * 
 * @see esphkSound
 */
typedef struct sSoundChunk {
    /**
     * 
     */
    unsigned long  sampleRate;
    /**
     *  sample  
     */
    unsigned long  sampleCount;
    /**
     *  
     */
    unsigned long  sampleSize;
    /**
     *   (6)
     */
    unsigned long  channelCount;
} sSoundChunk_t;

#endif							/* SLTMPACK_H */
